<?php

namespace Bixo\Locations\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Locations\Http\Requests\SublocationRequest;
use Bixo\Locations\Interfaces\SublocationRepositoryInterface;
use Bixo\Locations\Models\Sublocation;

/**
 * Resource controller class for sublocation.
 */
class SublocationResourceController extends BaseController
{

    /**
     * Initialize sublocation resource controller.
     *
     * @param type SublocationRepositoryInterface $sublocation
     *
     * @return null
     */
    public function __construct(SublocationRepositoryInterface $sublocation)
    {
        parent::__construct();
        $this->repository = $sublocation;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Locations\Repositories\Criteria\SublocationResourceCriteria::class);
    }

    /**
     * Display a list of sublocation.
     *
     * @return Response
     */
    public function index(SublocationRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Locations\Repositories\Presenter\SublocationPresenter::class)
                ->$function();
        }

        $sublocations = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('locations::sublocation.names'))
            ->view('locations::sublocation.index', true)
            ->data(compact('sublocations', 'view'))
            ->output();
    }

    /**
     * Display sublocation.
     *
     * @param Request $request
     * @param Model   $sublocation
     *
     * @return Response
     */
    public function show(SublocationRequest $request, Sublocation $sublocation)
    {

        if ($sublocation->exists) {
            $view = 'locations::sublocation.show';
        } else {
            $view = 'locations::sublocation.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('locations::sublocation.name'))
            ->data(compact('sublocation'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new sublocation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SublocationRequest $request)
    {

        $sublocation = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('locations::sublocation.name')) 
            ->view('locations::sublocation.create', true) 
            ->data(compact('sublocation'))
            ->output();
    }

    /**
     * Create new sublocation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SublocationRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $sublocation                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('locations::sublocation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('locations/sublocation/' . $sublocation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/locations/sublocation'))
                ->redirect();
        }

    }

    /**
     * Show sublocation for editing.
     *
     * @param Request $request
     * @param Model   $sublocation
     *
     * @return Response
     */
    public function edit(SublocationRequest $request, Sublocation $sublocation)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('locations::sublocation.name'))
            ->view('locations::sublocation.edit', true)
            ->data(compact('sublocation'))
            ->output();
    }

    /**
     * Update the sublocation.
     *
     * @param Request $request
     * @param Model   $sublocation
     *
     * @return Response
     */
    public function update(SublocationRequest $request, Sublocation $sublocation)
    {
        try {
            $attributes = $request->all();

            $sublocation->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('locations::sublocation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('locations/sublocation/' . $sublocation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('locations/sublocation/' . $sublocation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the sublocation.
     *
     * @param Model   $sublocation
     *
     * @return Response
     */
    public function destroy(SublocationRequest $request, Sublocation $sublocation)
    {
        try {

            $sublocation->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('locations::sublocation.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('locations/sublocation/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('locations/sublocation/' . $sublocation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple sublocation.
     *
     * @param Model   $sublocation
     *
     * @return Response
     */
    public function delete(SublocationRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('locations::sublocation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('locations/sublocation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/locations/sublocation'))
                ->redirect();
        }

    }

    /**
     * Restore deleted sublocations.
     *
     * @param Model   $sublocation
     *
     * @return Response
     */
    public function restore(SublocationRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('locations::sublocation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/locations/sublocation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/locations/sublocation/'))
                ->redirect();
        }

    }

}
